package kernel.peripherals;

import util.Color;

using tink.CoreApi;

typedef BundleMask = Color;

class Redstone implements IPeripheral {

	public final onChange:Signal<Noise>;

	private final onChangeTrigger:SignalTrigger<Noise> = Signal.trigger();

	private final addr:Side;

	private var analogInputState:Int;
	private var bundleInputState:BundleMask;

	@:allow(kernel.peripherals)
	private function new(side: Side) {
		this.addr = side;
		this.onChange = this.onChangeTrigger.asSignal();

		KernelEvents.instance.onRedstone.handle(()->{
			if ((this.getAnalogInput() != this.analogInputState) || (this.bundleInputState == this.getBundledInput())){
				this.onChangeTrigger.trigger(null);
			}
		});
	}

	public function getAddr():String {
		return this.addr;
		updateState();
	}

	private function updateState() {
		this.analogInputState = cc.Redstone.getAnalogInput(this.addr);
		this.bundleInputState = cc.Redstone.getBundledInput(this.addr);
	}

	public inline function setOutput(on:Bool):Void {
		this.analogInputState = 15;
		cc.Redstone.setOutput(this.addr,on);
	}

	public inline function getOutput(): Bool {
		return cc.Redstone.getOutput(this.addr);
	}

	public inline function getInput(): Bool {
		return cc.Redstone.getInput(this.addr);
	}

	public inline function setAnalogOutput(strength:Int): Void {
		this.analogInputState = strength;
		cc.Redstone.setAnalogOutput(this.addr,strength);
	}

	public inline function getAnalogOutput(): Int {
		return cc.Redstone.getAnalogOutput(this.addr);
	}

	public inline function getAnalogInput(): Int {
		return cc.Redstone.getAnalogInput(this.addr);
	}

	public inline function setBundledOutput(output: BundleMask) {
		this.bundleInputState = output;
		cc.Redstone.setBundledOutput(this.addr,output);
	}

	public inline function getBundledOutput():BundleMask {
		return cc.Redstone.getBundledOutput(this.addr);
	}

	public inline function getBundledInput():BundleMask {
		return cc.Redstone.getBundledInput(this.addr);
	}

	public inline function testBundledInput(mask: Color): Bool {
		return cc.Redstone.testBundledInput(this.addr,mask);
	}

}
